/*
 * Appcelerator Sync Server Example Application
 *
 * This application demonstrates how to use the Sync Server Client module to
 * synchronize MS Dynamics CRM data with the Sync Server.
 *
 * Before running this application, you must have an active Sync Server instance
 * with the msDynamicsCrm1.0 adapter configured and running.
 *
 * The Sync Server Client requires a `syncConfig.json` file with the appropriate
 * configuration options defined. The `syncConfig.json` file provided with this
 * example specifies a default `appId` and `serverAddress`. You must change the values
 * of these properties to match your Sync Server's configuration. You can either change
 * these properties in the `syncConfig.json` file OR set these properties on the
 * connector when you call `createSyncConnector`.
 *
 * The Sync Server requires a user and password for authentication. You must
 * update the credentials below before authenticating with the Sync Server
 *
 * To run the example:
 *   1. Install the appcelerator.syncserver.client module (if not already installed)
 *   2. Create a new Titanium Alloy application
 *   3. Add the appcelerator.syncserver.client module to your application's tiapp.xml file
 *   4. Copy the app folder from the example folder into your new Titanium Alloy application
 *   5. Update the appId and serverAddress properties in the syncConfig.json file
 *   6. Update the user and password credentials in the index.js file
 *   7. Run the application
 *   8. Click on 'Login' to verify that the credentials are correct
 *   9. Click on 'Logout' to verify that you are logged out
 *  10. Click on 'Sync' to pull the current records from the Sync Server
 *  11. Click on 'Queue changes' to update the local database and queue updates
 *  12. Click on 'Sync' to push and pull the records
 *
 * You can click on 'Queue changes' and 'Sync' multiple times to show the updating, deleting, and creating of
 * the sample contact record
 *
 * Progress messages are output to the console
 */

var connector = Alloy.Globals.syncConnector;

$.index.open();

var credentials = {
    user: 'user@company.com',
    password: 'password'
};

function formatJSON(evt) {
    return JSON.stringify(evt, function(key, value) {
        if(key === 'source' || key === 'type') {
            return undefined;
        } else {
            return value;
        }
    }, 2);
}

//=====================================================================

function login() {
    connector.login({
        user: credentials.user,
        password: credentials.password,
        success: function(e) {
            Ti.API.info('Successfully authenticated ' + connector.currentUser);
            Ti.API.info('SessionExpires: ' + e.sessionExpires);
        },
        error: function(e) {
            Ti.API.error('Error logging in: ' + e.message);
        },
        progress: function(e) {
            Ti.API.info("Login Progress: " + formatJSON(e));
        }
    });
}

function logout() {
    connector.logout({
        success: function (e) {
            Ti.API.info('Logged out');
        },
        error: function(e) {
            Ti.API.error('Error logging out: ' + e.message);
        },
        progress: function(e) {
            Ti.API.info("Logout Progress: " + formatJSON(e));
        }
    });
}

function queueChanges() {
    // Every record needs a unique identifier. This example application uses a
    // hard-coded GUID for demonstration purposes only. Normally your application
    // would generate a GUID for all new records that it creates.
    var id = '5c34e1e4-cfd2-e211-9dba-b4b52f67e641';
    
    // For demonstration purposes:
    //   If the record does not exist, create it
    //   If the record does exist and it has its initial value, update it
    //   Otherwise, delete it
    Alloy.Collections.contacts.fetch({
        success: function() {
            var contact = Alloy.Collections.contacts.where({contactid: id})[0];
    
            if (!contact) {
                Ti.API.info('Creating new contact record');
                Alloy.Collections.contacts.create({
                    contactid: id,
                    firstname: 'Peter',
                    lastname: 'Parker' 
                });
            } else {
                if (contact.get('lastname') === 'Parker') {
                    Ti.API.info('Updating existing contact record');
                    contact.save('lastname', 'Pan');
                } else {
                    Ti.API.info('Deleting existing contact record');
                    contact.destroy();
                }
            }
            
            Ti.API.info("Num: " + connector.getNumPendingChanges());
        }
    });
}

function synchronize() {
    connector.sync({
        user: credentials.user,
        password: credentials.password,
        success: function (e) {
            Ti.API.info("Successfully synchronized: " + formatJSON(e));
        },
        error: function (e) {
            Ti.API.error("Error during sync: " + formatJSON(e));
        },
        progress: function(e) {
            Ti.API.info("Sync Progress: " + formatJSON(e));
        }
    });
}

function cancel() {
    connector.cancel();
}